"""
Génération de QCM sous LaTeX
Auteur: Stéphane Pasquet
Date: 2023/09/06
Version: 1.00
URL: https://www.mathweb.fr/euclide/generer-un-qcm-a-laide-de-python-et-latex/
Syntaxe: qcm(file,colonnes(optionnel), solution=True|False (optionnel), colorcheck = '...')
    * file: chemin du fichier texte
    * colonnes: nombre de colonnes sur lesquelles répartir les réponses (par défaut: 1 colonne, donc 1 réponse par ligne)
    * solution = booléen (True|False). True = affiche les réponses correctes
    * colocheck = couleur de la coche quand les réponses correctes sont affichées. Par défaut, colorcheck = 'red'
Dans le fichier texte:
    * -NEWPAGE- pour sauter une page [version 1.01] (à insérer uniquement juste avant une question)
"""

import os.path

def qcm(file , colonnes = 1 , solution = False , colorcheck = 'red'):
    with open(file, 'r' , encoding='utf8') as qcm:
        Q = qcm.read().split('\n')

    titre = False
    niveau = False
    consigne = False
    question = False
    reponse = False
    newpage = False
    nq = 0
    
    with open('preambule.tex', 'r' , encoding='utf8') as fileTEX:
        latex = fileTEX.read()
    
    latex += '\n'
    
    for line in Q:
        match line:
            case '-NEWPAGE-':
                newpage = True
                latex += '\\end{multicols}\n\\end{enumerate} % fin des propositions de la question ' + str(nq) + '\n'
                latex += '\n\n\\newpage\n\\thispagestyle{fancy}\n\n'
            case 'T-':
                titre = True
                latex += '\\title{'
            case 'N-':
                niveau = True
                latex += '\\author{'
            case 'C-':
                consigne = True
                if solution == True:
                    latex += '\\begin{consigne}{Réponses}\n'
                else:
                    latex += '\\begin{consigne}{Consigne}\n'
                consigne = True
            case 'DEBUT-':
                latex += '\\begin{enumerate}\n'
            case 'Q-':
                if nq > 0:
                    if newpage == False:
                        if colonnes > 1:
                            latex += '\\end{multicols}\n'
                        latex += '\\end{enumerate} % fin des propositions de la question ' + str(nq) + '\n\n\\bigskip\n\n'
                    else:
                        newpage = False
                    reponse = False
                latex += '\\item {\\bfseries '
                question = True
                nq += 1
            case 'R-':
                latex += '\\begin{enumerate}\n'
                if colonnes > 1:
                    latex += '\\begin{multicols}{'+str(colonnes)+'}\n'
                reponse = True
            case 'FIN-':
                if colonnes > 1:
                    latex += '\\end{multicols}\n'
                latex += '\\end{enumerate} % fin de la question ' + str(nq) + '\n'
                latex += '\\end{enumerate} % fin du qcm\n\\end{document}'
            case _:
                if titre == True:
                    latex += line + '}\n'
                    titre = False
                elif niveau == True:
                    latex += line + '}\n'
                    niveau = False
                    latex += '\\date{\\today}\n'
                    latex += '\\maketitle\n\\thispagestyle{fancy}\n'
                elif consigne == True:
                    if solution == False:
                        latex += line+'\n\\end{consigne}\n % fin de la consigne\n'
                    else:
                        latex += 'Vous trouverez ci-dessous les réponses correctes.\n\\end{consigne}\n'
                    consigne = False
                elif question == True:
                    latex += line + '}\n'
                    question = False
                elif reponse == True:
                    if line != '':
                        P = line.split('|')
                        if int(P[1]) == 1 and solution == True:
                            latex += '\\item[\\checkbox{'+colorcheck+'}] '
                        else:
                            latex += '\\item[$\square$] '
                        latex += P[0] + '\n'
    
    if solution == True:
        ext = '-solution'
    else:
        ext = ''
        
    if os.path.exists(f'{file[:-4]}{ext}.tex'):
        os.remove(f'{file[:-4]}{ext}.tex')

    fichier = open(f'{file[:-4]}{ext}.tex' , "x" , encoding="utf8") # "x" pour la création et l'écriture
    fichier.write(latex)
    fichier.close()
    
    rep, fic = file.split('/')[0], file.split('/')[1]
    os.chdir(rep) # on se met dans le répertoire
    instructions = f'pdflatex "{fic[:-4]}{ext}.tex"'
    os.system(instructions)
    os.system(instructions)
    

    readpdf = f"START {fic[:-4]}{ext}.pdf"
    os.system(readpdf)